@@ -34,9 +34,12 @@
 #include <errno.h>
 #include <dirent.h>
 #include <sys/stat.h>
+#include <limits.h>
 
 #include <cpp/poppler-document.h>
 #include <cpp/poppler-page.h>
+#include <cpp/poppler-version.h>
+
 
 #ifdef HAVE_UNAC
 #include <unac.h>
@@ -70,6 +73,7 @@ int pagecount = 0;
 int quiet = 0;
 char *password = (char*)"";
 int max_count = 0;
+int debug = 0;
 
 #ifdef HAVE_UNAC
 int use_unac = 0;
@@ -87,6 +91,7 @@ enum {
 	EXCLUDE_OPTION,
 	INCLUDE_OPTION,
 	PASSWORD,
+	DEBUG_OPTION,
 #ifdef HAVE_UNAC
 	UNAC_OPTION,
 #endif
@@ -111,6 +116,7 @@ struct option long_options[] =
 	{"quiet", 0, 0, 'q'},
 	{"password", 1, 0, PASSWORD},
 	{"max-count", 1, 0, 'm'},
+	{"debug", 0, 0, DEBUG_OPTION},
 #ifdef HAVE_UNAC
 	{"unac", 0, 0, UNAC_OPTION},
 #endif
@@ -238,8 +244,8 @@ int search_in_document(poppler::document *doc, const std::string &filename, rege
 		simple_unac_free(unac_str);
 #endif
 		if(!quiet && pagecount && count_matches > page_matches) {
-			print_line_prefix(&outconf, filename.c_str(), -1);
-			printf("%d:%d\n", i, count_matches-page_matches);
+			print_line_prefix(&outconf, filename.c_str(), i);
+			printf("%d\n", count_matches-page_matches);
 			page_matches = count_matches;
 		}
 	}
@@ -393,13 +399,17 @@ void print_help(char *self)
 
 void print_version()
 {
-	printf("This is %s version %s\n", PACKAGE, VERSION);
-	if (strcmp(PDFGREP_GIT_HEAD, "") != 0) {
-		printf("Built from git-commit %s\n", PDFGREP_GIT_HEAD);
-	}
+	printf("This is %s version %s.\n", PACKAGE, VERSION);
+	printf("\nUsing poppler version %s\n", poppler::version_string().c_str());
+#ifdef HAVE_UNAC
+	printf("\nUsing unac version %s\n", unac_version());
+#endif
+#ifdef PDFGREP_GIT_HEAD
+	printf("Built from git-commit %s\n", PDFGREP_GIT_HEAD);
+#endif
 }
 
-int is_dir(const std::string filename)
+int is_dir(const std::string &filename)
 {
 	struct stat st;
 
@@ -409,7 +419,7 @@ int is_dir(const std::string filename)
 		return 0;
 }
 
-int do_search_in_document(const std::string path, const std::string filename,
+int do_search_in_document(const std::string &path, const std::string &filename,
 			  regex_t *ptrRegex, bool check_excludes = true)
 {
 	if (check_excludes &&
@@ -433,7 +443,7 @@ int do_search_in_document(const std::string path, const std::string filename,
 	return 0;
 }
 
-int do_search_in_directory(const std::string filename, regex_t *ptrRegex)
+int do_search_in_directory(const std::string &filename, regex_t *ptrRegex)
 {
 	DIR *ptrDir = NULL;
 	struct dirent *ptrDirent = NULL;
@@ -485,6 +495,29 @@ int do_search_in_directory(const std::string filename, regex_t *ptrRegex)
 	return 0;
 }
 
+bool parse_int(const char *str, int *i)
+{
+	char *endptr;
+	errno = 0;
+	unsigned long int d = strtoul(str, &endptr, 10);
+	if (errno != 0 || (long)d > INT_MAX || (long)d < INT_MIN
+	    || *endptr != '\0') {
+		return false;
+	}
+
+	*i = d;
+	return true;
+}
+
+#if POPPLER_VERSION_MAJOR > 0 || POPPLER_VERSION_MINOR >= 29
+void handle_poppler_errors(const std::string &msg, void *)
+{
+	if (debug) {
+		fprintf(stderr, "pdfgrep: %s\n", msg.c_str());
+	}
+}
+#endif
+
 int main(int argc, char** argv)
 {
 	regex_t regex;
@@ -536,12 +569,17 @@ int main(int argc, char** argv)
 				}
 				break;
 			case 'C':
-				if (optarg) {
-					if (!strcmp(optarg, "line")) {
-						context = -1;
-					} else {
-						context = atoi(optarg);
-					}
+				if (!strcmp(optarg, "line")) {
+					context = -1;
+					break;
+				}
+				if (!parse_int(optarg, &context)) {
+					fprintf(stderr, "pdfgrep: Could not parse number: %s.\n",
+						optarg);
+					exit(2);
+				} else if (context <= 0) {
+					fprintf(stderr, "pdfgrep: --context must be positive.\n");
+					exit(2);
 				}
 				break;
 			case EXCLUDE_OPTION:
@@ -552,8 +590,9 @@ int main(int argc, char** argv)
 				break;
 			case 'p':
 				pagecount = 1;
+				outconf.pagenum = 1;
 				break;
-			
+
 			case 'q':
 				quiet = 1;
 				break;
@@ -563,17 +602,30 @@ int main(int argc, char** argv)
 				break;
 
 			case 'm':
-				if (optarg) {
-					max_count = atoi(optarg);
+				if (!parse_int(optarg, &max_count)) {
+					fprintf(stderr, "pdfgrep: Could not parse number: %s.\n",
+						optarg);
+					exit(2);
+				} else if (max_count <= 0) {
+					fprintf(stderr, "pdfgrep: --max-count must be positive.\n");
+					exit(2);
 				}
 				break;
+			case DEBUG_OPTION:
+				debug = 1;
+				break;
 #ifdef HAVE_UNAC
 			case UNAC_OPTION:
 				use_unac = 1;
 				break;
 #endif
-			case '?':
-				/* TODO: do something here */
+
+			/* In these two cases, getopt already prints an
+			 * error message
+			 */
+			case '?': // unknown option
+			case ':': // missing argument
+				exit(2);
 				break;
 			default:
 				break;
@@ -598,6 +650,11 @@ int main(int argc, char** argv)
 		exit(2);
 	}
 
+#if POPPLER_VERSION_MAJOR > 0 || POPPLER_VERSION_MINOR >= 29
+	// set poppler error output function
+	poppler::set_debug_error_function(handle_poppler_errors, NULL);
+#endif
+
 	bool color_tty = isatty(STDOUT_FILENO) && getenv("TERM") &&
 		strcmp(getenv("TERM"), "dumb");
 	if (outconf.color == 1 && !color_tty) {
@@ -613,8 +670,12 @@ int main(int argc, char** argv)
 			outconf.filename = 1;
 	}
 
-	if (isatty(STDOUT_FILENO))
+	if (isatty(STDOUT_FILENO)) {
 		line_width = get_line_width();
+	} else if (context == -2) {
+		// on non-terminals, always print the whole line
+		context = -1;
+	}
 
 	if (excludes_empty(includes))
 		exclude_add(includes, "*.pdf");
